%%% This script enables the user to run an example from the paper. The
%%% example considers a range for the autocorrelation and a fixed variance ratio.

%%% This script enables the user to run POBE from the Matlab command line.
%%% When the run is finished input parameter are contained in the structure
%%% Inp and output parameter in the structure Out.
clear;

addpath('PATH') 
%PATH should specify the path of the program POBE

%%% specify your input parameters

%%% specify your design and effects of interest

bla = 15;   
%task block length in seconds
bln = 15;   
%null block length in seconds
Inp.TR = 2.5;   
%repetition time in seconds
Inp.SOA = 2.5;  
%stimulus onset asynchrony in seconds
Inp.Q = 1;      
%number of stimulus types
Inp.blockorder = 1; 
%1=ABN,2=ANBN
Inp.C = eye(Inp.Q); 
%specify effects of interest 

%%% specify your optimality criterion
Inp.optcrit = 1;    
%1=A-criterion, 2=D-criterion 

%%% specify your costs 
Inp.C1 = 200; 
%subjectCosts   
Inp.C2 = 400; 
%scanning costs per hour
Inp.CT = 6000; 
%total study budget

%%% specify your autocorrelation and variance parameter
Inp.previousAuto = 1;
%1 = specify autocorrelation, 
%2 = choose autocorrelation from previous study
%If option 2 is chosen, a file autocorr.mat has to be specified. This file should containin a vector with cross-correlations
%over several time points from a previous study.
%If option 2 is chosen no specification of rho is necessary below.

%Inp.rho = 0.25;          
%fixed value for autocorrelation parameter 
Inp.minRho = 0.12;         
%minimum value for range of autocorrelation parameter
Inp.maxRho = 0.33;       
%maximum value for range for autocorrelation parameter
Inp.varRat = 6.16;         
%fixed value for variance ratio
%Inp.minVarRat = 2.03;    
%minimum value for range of variance ratio
%Inp.maxVarRat = 13.69;     
%maximum value for range of variance ratio
Inp.corrRE = 0;         
%random effects correlation
%should be left to 0 even if not relevant for further calculations

%depending on option chosen for valAR1VarRat it is not necessary to specify
%some of the parameters above (rho,minRho,maxRho,varRat,minVarRat,maxVarRat)
valAR1VarRat = 2;  
%1 = autcorrelation and variance ratio fixed
%2 = range for autocorrelation (variance ratio fixed)
%3 = range for variance ratio (autocorrelation fixed)
%4 = range for autocorrelation (variance ratio fixed) and range for variance ratio (autcorrelation fixed)
%5 = range for autocorrelation and variance ratio (nothing fixed) 

%if Inp.autoPrevious = 2 is chosen, option 2, 4 and 5 do not make sense

%%%% specify your nuisance terms
Inp.opt = 'Discrete cosine transform'; 
% 'Legendre' or 'Discrete cosine transforms'
Inp.order = 3; 
%order of nuisance terms 

%%% specify user defined HRF or keep standard parameters
shape = 'twogamma';         
%see hrf.m for further information
pHRF = [0.1,5,15,6,0,1,1];  
%see hrf.m for further information
s = [0,32];                 
%see hrf.m for further information    

scale = 1; 
%scaling parameter to model nonlinear saturation of HRF

%%% additional outcome parameters
valMMDAR1 = 1;      
%0 = calculate no maximin design for autocorrelation range
%1 = calculate maximin design 

valMMDVarRat = 0;   
%0 = calculate no maximin design for variance ratio range
%1 = calculate maximin design for variance ratio range

power = 0; 
%is executed if possible
%0 = calculate no power
%1 = calculate power
%Inp.effectSize = 0.5;     %effect size
%Inp.sigmaEps2 = 2.464;      %within-subject variance
%Inp.sigmaQ2 = 0.4;        %between-subject variance
%Inp.alpha = 0.005;       %significance level
%please check that Inp.sigmaEp2/Inp.sigmQ2 = Inp.varRat

%%%%%%%% calcuation of optimal number of subjects and cycles and maximin designs (do not change) %%%%%%%%%%
disp('Please wait: calculation is running')
Inp.dT = gcd_new(Inp.TR,Inp.SOA);

Inp.runtime = 'none';
Inp.nbln = bln/Inp.SOA;
Inp.nbla = bla/Inp.SOA;
Inp.pround = Inp.nbla/(Inp.Q*Inp.nbla+Inp.nbln);

Inp.costsRat = Inp.C1/(Inp.C2/3600);

%autocorrelation and variance parameter
Inp.D=sptoeplitz([1 repmat(Inp.corrRE,1,Inp.Q-1)]);      

%calculation of HRF 
%HRF is sampled over lengthHRF with frequency pHRF(1)
Inp.lengthHRF = s(2)-s(1);
h = hrf(shape,pHRF(1),pHRF(2),pHRF(3),pHRF(4),pHRF(5),pHRF(6),pHRF(7),s,0);
h = scale * h / max(h); % scale HRF to have a max of 1 and modify eventually with scale

% downsampling of HRF
h = h(1:(Inp.dT*1/pHRF(1)):end,:);
Inp.Hmat=kron(eye(Inp.Q),h);
Inp.nrtpHRF=floor(Inp.lengthHRF/Inp.dT)+1; 

switch valAR1VarRat
    case 1
        if (Inp.pround~=1/Inp.Q) && (Inp.TR==Inp.SOA) && (Inp.rho~=0) && ...
                (Inp.order ==0) && (Inp.blockorder == 1)
            [NCmin N]=findOptNCandNun(Inp);
            Inp.NCmin=max(1,floor(NCmin));
        else 
            Inp.NCmin=1;
        end
        Inp.NCmin = 1;
        [optNC optN] = findOptNCandNAR1(Inp);
        Out.NC = optNC;
        Out.N = optN;
        if Inp.blockorder == 1
            Out.costs = optN*Inp.C1+optN*optNC*Inp.C2/3600*(Inp.Q*bla+bln);
            Out.time = optNC*(Inp.Q*bla+bln)/60;
        else
            Out.costs = optN*Inp.C1+optN*optNC*Inp.C2/3600*Inp.Q*(bla+bln);
            Out.time = optNC*Inp.Q*(bla+bln)/60;   
        end    
        if power == 1 
            if Inp.Q == 1 || size(Inp.C,1)== 1
                Out.power = calc_power(Inp,optNC,optN);
            end    
        end    
    case 2
        [optNC optN] = findOptNCandNAR1loopAR1(Inp);
        Out.optNC = optNC;
        Out.optN = optN;
        if valMMDAR1 == 1
            [NCmax Nmax maxMinRe] = maximinAR1(Inp,optNC,optN);
            Out.NCmaximin = NCmax;
            Out.Nmaximin = Nmax;
            Out.maxMinRe = maxMinRe;
            if Inp.blockorder == 1
                Out.costs =Nmax*Inp.C1+Nmax*NCmax*Inp.C2/3600*(Inp.Q*bla+bln);
                Out.time = NCmax*(Inp.Q*bla+bln)/60;
            else
                Out.costs = Nmax*Inp.C1+Nmax*NCmax*Inp.C2/3600*Inp.Q*(bla+bln);
                Out.time = NCmax*Inp.Q*(bla+bln)/60;   
            end
        end    
     case 3
        [optNC optN] = findOptNCandNAR1loopVarRat(Inp);
        Out.optNC = optNC;
        Out.optN = optN;
        if valMMDVarRat == 1
            [NCmax Nmax maxMinRe] = maximinVarRat(Inp,optNC,optN);
            Out.NCmaximin = NCmax;
            Out.Nmaximin = Nmax;
            Out.maxMinRe = maxMinRe;
            if Inp.blockorder == 1
                Out.costs =Nmax*Inp.C1+Nmax*NCmax*Inp.C2/3600*(Inp.Q*bla+bln);
                Out.time = NCmax*(Inp.Q*bla+bln)/60;
            else
                Out.costs = Nmax*Inp.C1+Nmax*NCmax*Inp.C2/3600*Inp.Q*(bla+bln);
                Out.time = NCmax*Inp.Q*(bla+bln)/60;   
            end              
        end    
    case 4
        [optNCAR1loop optNAR1loop] = findOptNCandNAR1loopAR1(Inp);
        Out.optNC_correlation_range = optNCAR1loop;
        Out.optN_correlation_range =optNAR1loop;
        [optNCVarRatloop optNVarRatloop] = findOptNCandNAR1loopVarRat(Inp);
        Out.optNC_variance_ratio_range = optNCVarRatloop;
        Out.optN_variance_ratio_range =optNVarRatloop;
        if valMMDAR1 == 1
            [NCmaxAR1 NmaxAR1 maxMinReAR1] = maximinAR1(Inp,optNCAR1loop,optNAR1loop);
            Out.NCmaximin_correlation_range = NCmaxAR1;
            Out.Nmaximin_correlation_range = NmaxAR1;
            Out.maxMinRE_correlation_range = maxMinReAR1;
            if Inp.blockorder == 1
                Out.costs_correlation_range = NmaxAR1*Inp.C1+NmaxAR1*NCmaxAR1*Inp.C2/3600*(Inp.Q*bla+bln);
                Out.time_correlation_range = NCmaxAR1*(Inp.Q*bla+bln)/60;
            else
                Out.costs_correlation_range = NmaxAR1*Inp.C1+NmaxAR1*NCmaxAR1*Inp.C2/3600*Inp.Q*(bla+bln);
                Out.time_correlation_range = NCmaxAR1*Inp.Q*(bla+bln)/60;   
            end
        end    
        if valMMDVarRat == 1
            [NCmaxVarRat NmaxVarRat maxMinReVarRat] = maximinVarRat(Inp,optNCVarRatloop,optNVarRatloop);
            Out.NCmaximin_variance_ratio_range = NCmaxVarRat;
            Out.Nmaximin_variance_ratio_range = NmaxVarRat;
            Out.maxMinRE_variance_ratio_range = maxMinReVarRat;
            if Inp.blockorder == 1
                Out.costs_variance_ratio_range =NmaxVarRat*Inp.C1+NmaxVarRat*NCmaxVarRat*Inp.C2/3600*(Inp.Q*bla+bln);
                Out.time_variance_ratio_range = NCmaxVarRat*(Inp.Q*bla+bln)/60;
            else
                Out.costs_variance_ratio_range = NmaxVarRat*Inp.C1+NmaxVarRat*NCmaxVarRat*Inp.C2/3600*Inp.Q*(bla+bln);
                Out.time_variance_ratio_range = NCmaxVarRat*Inp.Q*(bla+bln)/60;   
            end
        end
    case 5
        [optNCAR1VarRatloop optNAR1VarRatloop] = findOptNCandNAR1loopAR1VarRat(Inp);
        Out.optNC = optNCAR1VarRatloop;
        Out.optN = optNAR1VarRatloop;
        [NCmaxAR1VarRat NmaxAR1VarRat maxMinReAR1VarRat] = maximinAR1VarRat(Inp,optNCAR1VarRatloop,optNAR1VarRatloop);
        Out.NCmaximin = NCmaxAR1VarRat;
        Out.Nmaximin = NmaxAR1VarRat;
        Out.maxMinRe = maxMinReAR1VarRat;
        
        if Inp.blockorder == 1
            Out.costs =NmaxAR1VarRat*Inp.C1+NmaxAR1VarRat*NCmaxAR1VarRat*Inp.C2/3600*(Inp.Q*bla+bln);
            Out.time = NCmaxAR1VarRat*(Inp.Q*bla+bln)/60;
        else
            Out.costs = NmaxAR1VarRat*Inp.C1+NmaxAR1VarRat*NCmaxAR1VarRat*Inp.C2/3600*Inp.Q*(bla+bln);
            Out.time = NCmaxAR1VarRat*Inp.Q*(bla+bln)/60;   
        end
end
disp('Calculation is finished')
Out
