%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this function finds optimal number of cycles and optimal number of
% subjects for AR1 error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [optNC optN] = findOptNCandNAR1(Inp)

%preparations which can be done outside of loop
if Inp.blockorder == 1
    seqBase=ones(1,Inp.nbla)'*(1:Inp.Q);
    seqBase=seqBase(:);
    seqBase=[seqBase;zeros(Inp.nbln,1)];
else   
    seqBase=[ones(1,Inp.nbla) zeros(1,Inp.nbln)]'*(1:Inp.Q);
    seqBase=seqBase(:);
end    
seq = repmat(seqBase,Inp.NCmin-1,1);
trCDCT = sum(diag(Inp.C*Inp.D*Inp.C'));

found = 0;
foundInt = 0;
NC = Inp.NCmin-1;
if Inp.blockorder == 1
    N = Inp.CT/(Inp.C1+NC*(Inp.Q*Inp.nbla+Inp.nbln)*Inp.SOA*Inp.C2/3600);
    Nint = round(Inp.CT/(Inp.C1+NC*(Inp.Q*Inp.nbla+Inp.nbln)*Inp.SOA*Inp.C2/3600));
    if (Nint*Inp.C1+Nint*NC*(Inp.Q*Inp.nbla+Inp.nbln)*Inp.SOA*Inp.C2/3600>Inp.CT)
        Nint=Nint-1;
    end
else    
    N = Inp.CT/(Inp.C1+NC*Inp.Q*(Inp.nbla+Inp.nbln)*Inp.SOA*Inp.C2/3600);
    Nint = round(Inp.CT/(Inp.C1+NC*Inp.Q*(Inp.nbla+Inp.nbln)*Inp.SOA*Inp.C2/3600));
    if (Nint*Inp.C1+Nint*NC*Inp.Q*(Inp.nbla+Inp.nbln)*Inp.SOA*Inp.C2/3600>Inp.CT)
        Nint=Nint-1;
    end
end    
j = 0;
tr = +inf; %initialisation of value of trace for unrounded N
tr_round = +inf; %initialisation of value of trace for rounded N
while (found == 0) || (foundInt ~= 1)   
    trPrev = tr;
    trPrev_round = tr_round;
    
    Nprev = N;
    Nprevint=Nint;
    j=j+1;
    NC = NC+1;

    %numScan is number of Scans
    if Inp.blockorder == 1
        numScan = floor(NC*(Inp.Q*Inp.nbla+Inp.nbln)*Inp.SOA/Inp.TR);
    else
        numScan = floor(NC*Inp.Q*(Inp.nbla+Inp.nbln)*Inp.SOA/Inp.TR);
    end    
    %create stimulus sequence (time between events is SOA)
    seq = [seq;seqBase];

    deconvMfinal = DconvMTX(seq, numScan, Inp.Q, Inp.lengthHRF, Inp.SOA, Inp.TR, Inp.dT);

    Z=deconvMfinal*Inp.Hmat;
    Smat = highpassFilter(numScan,Inp.order,Inp.opt);
    if Inp.previousAuto == 1
        base = [1+Inp.rho^2, -Inp.rho, zeros(1, numScan-2)]; 
        V2 = sptoeplitz(base);
        V2(1,1) = 1;
        V2(numScan,numScan) = 1; 
        V2=1/(1-Inp.rho^2).*V2; %square of whitening matrix=inverse of error covariance matrix
    else
        load('autocorr.mat')
        V2 = getv('make',myscannerxc,numScan);
    end
    helpM = V2*Smat;
    whiteM = V2 - helpM/(Smat'*helpM)*helpM'; %V(I-P)V
    
    product=Z'*whiteM*Z;
     
    N = Inp.CT/(Inp.C1+numScan*Inp.TR*Inp.C2/3600);
    Nint = round(Inp.CT/(Inp.C1+numScan*Inp.TR*Inp.C2/3600)); %rounded N;
    %note: floor did not work because for some combinations
    %Nvecint=x.99999 instead of Nvecint=x+1 due to rounding problems
    %thus floor would take x which is too low
    
    %if Nvecint(j) too high, take next smallest integer
    if (Nint*Inp.C1+Nint*numScan*Inp.TR*Inp.C2/3600>Inp.CT)
        Nint=Nint-1;
    end
    
    if Inp.optcrit == 1
        tr=(Inp.varRat*sum(diag(Inp.C*product^(-1)*Inp.C'))+trCDCT)/N;
        tr_round=(Inp.varRat*sum(diag(Inp.C*product^(-1)*Inp.C'))+trCDCT)/Nint;
    else    
        tr = det(Inp.varRat*Inp.C*product^(-1)*Inp.C'+Inp.C*Inp.D*Inp.C')/N;
        tr_round = det(Inp.varRat*Inp.C*product^(-1)*Inp.C'+Inp.C*Inp.D*Inp.C')/Nint;
    end
    if (tr>trPrev)&&(found == 0)
    %disp('optimal NC and N for unrounded N found')
        found = 1;
        NCopt = NC-1;
        Nopt = Nprev;
    end    
    if (tr_round>trPrev_round) && (foundInt == 0)
        %disp('first candidate found')
        foundInt = 0.5; %we have found a candidante for best NCoptint and Noptint
        NCoptintcand = NC-1;
        Noptintcand = Nprevint;
        tropt_roundcand = trPrev_round;
    end    
    if (foundInt == 0.5) && (tr_round < tropt_roundcand)
        %disp('new candidate found')
        %previous candidate has failed, we have found new candidate    
        NCoptintcand = NC;
        Noptintcand = Nint;
        tropt_roundcand = tr_round;
    end
    if (foundInt == 0.5) && (tr > tropt_roundcand)
    %candidate is chosen as best solution   
    %explanation second condition: if tr of this NC is higher than tropt_roundcand, all 
    %tr_round for this NC or higher NC can only be higher than
    %tropt_roundcand because tr_round > tr for all NC
        %disp('candidate chosen')
        foundInt = 1;
        NCoptint = NCoptintcand;
        Noptint = Noptintcand;
    end    
end

optNC = NCoptint;
optN = Noptint;
%optNC = NCopt;
%optN = Nopt;