%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this function finds the optimal number of cycles and subjects for
% uncorrelated errors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [optNC optN]=findOptNCandNun(Inp)
%see Maus et al, NeuroImage 56 (2011) 13381352

%calculation of M according to Maus et al (2011)
nblt = Inp.Q*Inp.nbla + Inp.nbln;
nblnt = (Inp.Q-1)*Inp.nbla + Inp.nbln;
prodTheo = zeros(Inp.nrtpHRF,Inp.Q*Inp.nrtpHRF);
for k=1:Inp.Q
   for i=1:Inp.nrtpHRF
       for j=1:Inp.nrtpHRF
           shift=mod(abs(j-i+(k-1)*Inp.nbla),nblt); 
           prodTheo(i,j+(k-1)*Inp.nrtpHRF)=(1-Inp.pround)^2*abs(max(Inp.nbla - shift,0)+max(0,shift-nblnt)) + (-Inp.pround)^2 * (max(nblnt - shift,0)+max(0,shift-Inp.nbla)) + 2* (-Inp.pround)*(1-Inp.pround)*min([shift nblt-shift Inp.nbla]);
       end
   end
end
for k=2:Inp.Q
   for l=1:Inp.Q
       if k<=l
           prodTheo((k-1)*Inp.nrtpHRF+1:k*Inp.nrtpHRF,(l-1)*Inp.nrtpHRF+1:l*Inp.nrtpHRF)=prodTheo(1:Inp.nrtpHRF,abs(k-l)*Inp.nrtpHRF+1:(abs(k-l)+1)*Inp.nrtpHRF);
       elseif k>l
           prodTheo((k-1)*Inp.nrtpHRF+1:k*Inp.nrtpHRF,(l-1)*Inp.nrtpHRF+1:l*Inp.nrtpHRF)=prodTheo(1:Inp.nrtpHRF,abs(k-l)*Inp.nrtpHRF+1:(abs(k-l)+1)*Inp.nrtpHRF)';
       end
   end
end

M=Inp.Hmat'*prodTheo*Inp.Hmat;
invM=M^(-1);

trContInvM=sum(diag(Inp.C*invM*Inp.C'));

%note D is here random effects correlation matrix
trCDCT=sum(diag(Inp.C*Inp.D*Inp.C'));    
optNC=sqrt(Inp.costsRat*Inp.varRat/trCDCT)*sqrt(trContInvM/(Inp.TR*nblt));
optN=Inp.CT/(Inp.C1+sqrt(Inp.C1*Inp.C2/3600*Inp.varRat/trCDCT)*sqrt(trContInvM*Inp.TR*nblt));
